@@ -1,6 +1,6 @@
 #!/bin/sh
 
-unzip -o S-20181019.zip
+unzip -o S-34.zip
 
 echo "#!/bin/sh
 
@@ -29,10 +29,9 @@ cd S-master
 if [ ! \"X\$6\" = \"X\" ]
 then
 echo Invoking special dir >\$mylog
-sed -i \"s<.*BASE_DIR=.*<BASE_DIR=\$3<\" def_config_params.sh >>\$mylog 2>&1
+sed -i \"s<.*BASE_DIR=.*<BASE_DIR=\$3<\" def_config.sh >>\$mylog 2>&1
 else
 echo Invoking local dir >\$mylog
-sed -i 's<.*BASE_DIR=.*<BASE_DIR=\$PWD/../workfiles<' def_config_params.sh >>\$mylog 2>&1
 fi
 
 cd comm_startup_lat
