@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2011 - 2013, Phoronix Media
-	Copyright (C) 2011 - 2013, Michael Larabel
+	Copyright (C) 2011 - 2014, Phoronix Media
+	Copyright (C) 2011 - 2014, Michael Larabel
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@ class download_test_files implements pts_option_interface
 		if(count($test_profiles) > 0)
 		{
 			echo PHP_EOL . 'Downloading Test Files For: ' . implode(' ', $test_profiles);
-			pts_test_installer::only_download_test_files($test_profiles, pts_client::read_env('DOWNLOAD_CACHE_LOCATION'));
+			pts_test_installer::only_download_test_files($test_profiles, pts_env::read('PTS_DOWNLOAD_CACHE'));
 		}
 		else
 		{
