@@ -39,9 +39,43 @@ class phodevi_gpu extends phodevi_device_interface
 			'screen-resolution' => new phodevi_device_property('gpu_screen_resolution', phodevi::std_caching),
 			'screen-resolution-string' => new phodevi_device_property('gpu_screen_resolution_string', phodevi::std_caching),
 			'2d-acceleration' => new phodevi_device_property('gpu_2d_acceleration', phodevi::std_caching),
+			'bar1-visible-vram' => new phodevi_device_property('bar1_visible_vram', phodevi::smart_caching),
 			'device-id' => new phodevi_device_property('gpu_pci_device_id', phodevi::smart_caching),
 			);
 	}
+	public static function bar1_visible_vram()
+	{
+		$rebar_size = '';
+		if(phodevi::is_nvidia_graphics())
+		{
+			if(pts_client::executable_in_path('nvidia-smi'))
+			{
+				$nvidia_smi = shell_exec('nvidia-smi -a | grep BAR -A 3');
+
+				if(($x = strpos($nvidia_smi, 'Total ')) !== false)
+				{
+					$nvidia_smi = substr($nvidia_smi, $x);
+					$nvidia_smi = substr($nvidia_smi, strpos($nvidia_smi, ': ') + 2);
+					$nvidia_smi = substr($nvidia_smi, 0, strpos($nvidia_smi, PHP_EOL));
+					$rebar_size = $nvidia_smi;
+				}
+			}
+		}
+		if(phodevi::is_linux() && pts_client::executable_in_path('glxinfo'))
+		{
+			// Could improve detection above to only AMD Radeon graphics...
+			$amd_debug_glxinfo = shell_exec('AMD_DEBUG=info glxinfo 2>&1 | grep vram');
+
+			if($amd_debug_glxinfo != null && ($x = strpos($amd_debug_glxinfo, 'vram_vis_size = ')) !== false)
+			{
+				$amd_debug_glxinfo = substr($amd_debug_glxinfo, $x + 16);
+				$amd_debug_glxinfo = substr($amd_debug_glxinfo, 0, strpos($amd_debug_glxinfo, PHP_EOL));
+				$rebar_size = $amd_debug_glxinfo;
+			}
+		}
+
+		return $rebar_size;
+	}
 	public static function gpu_pci_device_id()
 	{
 		$device_id = null;
@@ -642,7 +676,7 @@ class phodevi_gpu extends phodevi_device_interface
 		else
 		{
 			// Try reading video memoty from GLX_MESA_query_renderer output in glxinfo
-			$glxinfo_video_mem = str_replace('MB', null, phodevi_parser::glxinfo_read_line('Video memory'));
+			$glxinfo_video_mem = str_replace('MB', '', phodevi_parser::glxinfo_read_line('Video memory'));
 
 			if(is_numeric($glxinfo_video_mem) && $glxinfo_video_mem > 1)
 			{
@@ -805,7 +839,7 @@ class phodevi_gpu extends phodevi_device_interface
 				$core_string = array_search('core', $performance_level);
 				if($core_string !== false && isset($performance_level[($core_string + 1)]))
 				{
-					$core_string = str_ireplace('MHz', null, $performance_level[($core_string + 1)]);
+					$core_string = str_ireplace('MHz', '', $performance_level[($core_string + 1)]);
 					if(is_numeric($core_string))
 					{
 						$core_freq = $core_string;
@@ -814,7 +848,7 @@ class phodevi_gpu extends phodevi_device_interface
 				$mem_string = array_search('memory', $performance_level);
 				if($mem_string !== false && isset($performance_level[($mem_string + 1)]))
 				{
-					$mem_string = str_ireplace('MHz', null, $performance_level[($mem_string + 1)]);
+					$mem_string = str_ireplace('MHz', '', $performance_level[($mem_string + 1)]);
 					if(is_numeric($mem_string))
 					{
 						$mem_freq = $mem_string;
@@ -846,7 +880,7 @@ class phodevi_gpu extends phodevi_device_interface
 				$core_string = array_search('core', $performance_level);
 				if($core_string !== false && isset($performance_level[($core_string + 1)]))
 				{
-					$core_string = str_ireplace('MHz', null, $performance_level[($core_string + 1)]);
+					$core_string = str_ireplace('MHz', '', $performance_level[($core_string + 1)]);
 					if(strpos($core_string, '-') !== false)
 					{
 						// to work around a range of values, e.g.
@@ -861,7 +895,7 @@ class phodevi_gpu extends phodevi_device_interface
 				$mem_string = array_search('memory', $performance_level);
 				if($mem_string !== false && isset($performance_level[($mem_string + 1)]))
 				{
-					$mem_string = str_ireplace('MHz', null, $performance_level[($mem_string + 1)]);
+					$mem_string = str_ireplace('MHz', '', $performance_level[($mem_string + 1)]);
 					if(strpos($mem_string, '-') !== false)
 					{
 						// to work around a range of values, e.g.
@@ -1121,7 +1155,7 @@ class phodevi_gpu extends phodevi_device_interface
 				if(($x = strpos($xorg_log, '(0): Chipset: ')) !== false)
 				{
 					$xorg_log = substr($xorg_log, ($x + 14));
-					$xorg_log = str_replace(array('(R)', '"'), null, substr($xorg_log, 0, strpos($xorg_log, PHP_EOL)));
+					$xorg_log = str_replace(array('(R)', '"'), '', substr($xorg_log, 0, strpos($xorg_log, PHP_EOL)));
 
 					if(($c = strpos($xorg_log, '[')) || ($c = strpos($xorg_log, '(')))
 					{
@@ -1187,7 +1221,7 @@ class phodevi_gpu extends phodevi_device_interface
 			{
 				unset($windows_gpu[$x]);
 			}
-			$info = str_replace('(TM)', null, implode(' + ', $windows_gpu));
+			$info = str_replace('(TM)', '', implode(' + ', $windows_gpu));
 		}
 
 		if(empty($info) || strpos($info, 'Mesa ') !== false || strpos($info, 'Gallium ') !== false || strpos($info, 'DRM ') !== false)
@@ -1200,14 +1234,14 @@ class phodevi_gpu extends phodevi_device_interface
 			if(strpos($info, 'Intel ') !== false)
 			{
 				// Intel usually has e.g. TGL GT2 or other info within
-				$info = str_replace(array('(', ')'), null, $info);
+				$info = str_replace(array('(', ')'), '', $info);
 			}
 			if(($x = strpos($info, ' (')) !== false)
 			{
 				$info = substr($info, 0, $x);
 			}
 
-			if(phodevi::is_windows() == false && strpos($info, 'Intel ') === false)
+			if(phodevi::is_windows() == false && strpos($info, 'Intel ') === false && !pts_strings::string_contains($info, pts_strings::CHAR_NUMERIC))
 			{
 				$info_pci = phodevi_linux_parser::read_pci('VGA compatible controller', false);
 
@@ -1233,7 +1267,7 @@ class phodevi_gpu extends phodevi_device_interface
 							if(($x = strpos($xorg_log, 'Integrated Graphics Chipset: ')) !== false)
 							{
 								$xorg_log = substr($xorg_log, ($x + 29));
-								$xorg_log = str_replace(array('(R)', '"'), null, substr($xorg_log, 0, strpos($xorg_log, PHP_EOL)));
+								$xorg_log = str_replace(array('(R)', '"'), '', substr($xorg_log, 0, strpos($xorg_log, PHP_EOL)));
 
 								if(stripos($xorg_log, 'Intel') === false)
 								{
@@ -1250,7 +1284,7 @@ class phodevi_gpu extends phodevi_device_interface
 							else if(($x = strpos($xorg_log, '(0): Chipset: ')) !== false)
 							{
 								$xorg_log = substr($xorg_log, ($x + 14));
-								$xorg_log = str_replace(array('(R)', '"'), null, substr($xorg_log, 0, strpos($xorg_log, PHP_EOL)));
+								$xorg_log = str_replace(array('(R)', '"'), '', substr($xorg_log, 0, strpos($xorg_log, PHP_EOL)));
 
 								if(stripos($xorg_log, 'Intel') === false)
 								{
@@ -1408,7 +1442,7 @@ class phodevi_gpu extends phodevi_device_interface
 		}
 
 		$clean_phrases = array('OpenGL Engine');
-		$info = str_replace($clean_phrases, null, $info);
+		$info = str_replace($clean_phrases, '', $info);
 
 		if(!empty($info) && $video_ram > 64 && strpos($info, $video_ram) == false && stripos($info, 'llvmpipe') === false) // assume more than 64MB of vRAM
 		{
