@@ -55,7 +55,7 @@ class phodevi_motherboard extends phodevi_device_interface
 		}
 		else if(phodevi::is_windows())
 		{
-			$confirm = shell_exec('powershell "Confirm-SecureBootUEFI"');
+			$confirm = shell_exec('powershell -NoProfile "Confirm-SecureBootUEFI"');
 			if(strpos($confirm, 'True') !== false)
 			{
 				$status = 'Enabled';
@@ -133,7 +133,7 @@ class phodevi_motherboard extends phodevi_device_interface
 				}
 
 				$vendor = pts_strings::trim_search_query(pts_strings::strip_string(pts_file_io::file_get_contents($usb_dir . 'manufacturer')));
-				$device = pts_strings::trim_search_query(pts_strings::strip_string(str_replace($vendor, null, pts_file_io::file_get_contents($usb_dir . 'product'))));
+				$device = pts_strings::trim_search_query(pts_strings::strip_string(str_replace($vendor, '', pts_file_io::file_get_contents($usb_dir . 'product'))));
 				$device = pts_strings::keep_in_string($device, pts_strings::CHAR_LETTER | pts_strings::CHAR_NUMERIC | pts_strings::CHAR_DECIMAL | pts_strings::CHAR_SPACE | pts_strings::CHAR_DASH | pts_strings::CHAR_UNDERSCORE | pts_strings::CHAR_COLON | pts_strings::CHAR_COMMA);
 
 				if($vendor == null || $device == null || $vendor == 'Generic')
@@ -277,7 +277,7 @@ class phodevi_motherboard extends phodevi_device_interface
 
 			$device_name = substr($device, ($l = strpos($device, ']:') + 3), ($s = strpos($device, ':', $l)) - $l);
 			$device_name = substr($device_name, 0, strrpos($device_name, ' ['));
-			$device_name = str_replace('/', '-', str_replace(array('[AMD]', '[SiS]'), null, $device_name));
+			$device_name = str_replace('/', '-', str_replace(array('[AMD]', '[SiS]'), '', $device_name));
 			$device_name = pts_strings::strip_string($device_name);
 
 			if($device_name == null || strpos($device_name, ' ') === false)
@@ -313,7 +313,7 @@ class phodevi_motherboard extends phodevi_device_interface
 				{
 					foreach(explode(' ', trim($temp)) as $temp)
 					{
-						$temp = str_replace(',', null, $temp);
+						$temp = str_replace(',', '', $temp);
 						if($temp != null && !in_array($temp, $drivers))
 						{
 							array_push($drivers, $temp);
@@ -453,7 +453,7 @@ class phodevi_motherboard extends phodevi_device_interface
 			$bios_version = null;
 		}
 
-		return trim(str_replace(array('(', ')'), '', $bios_version));
+		return empty($bios_version) ? '' : trim(str_replace(array('(', ')'), '', $bios_version));
 	}
 	public static function motherboard_string()
 	{
@@ -480,7 +480,7 @@ class phodevi_motherboard extends phodevi_device_interface
 			$product = phodevi_bsd_parser::read_kenv('smbios.system.product');
 			$version = phodevi_bsd_parser::read_kenv('smbios.system.version'); // for at least Lenovo ThinkPads this is where it displays ThinkPad model
 
-			if($vendor != null && ($product != null || $version != null) && pts_strings::has_alpha($vendor) && strpos($product, 'System') === false)
+			if($vendor != null && ($product != null || $version != null) && strpos($product, 'System') === false)
 			{
 				$info = $vendor . ' ' . $product . ' ' . $version;
 			}
@@ -512,6 +512,30 @@ class phodevi_motherboard extends phodevi_device_interface
 				{
 					$info .= (substr($version, 0, 1) == 'v' ? ' ' : ' v') . $version;
 				}
+
+				if((phodevi::is_root() || is_readable('/dev/mem')) && pts_client::executable_in_path('dmidecode'))
+				{
+					// For some vendors, it's better to read system-product-name
+					// Unfortunately other vendors report garbage here, also demidecode only works as root on Linux
+					foreach(array('Dell', 'Apple') as $vend)
+					{
+						if(stripos($info, $vend . ' ') !== false)
+						{
+							$dmi_output = shell_exec('dmidecode -s system-product-name 2>&1');
+							if($dmi_output != null && stripos($dmi_output, ' ') !== false && stripos($dmi_output, 'invalid') === false && stripos($dmi_output, 'System Product') === false && stripos($dmi_output, 'not ') === false)
+							{
+								$old_info = trim(str_ireplace(array($vend . ' ', 'Inc.'), '', $info));
+								$info = trim($dmi_output) . (!empty($old_info) && strpos($dmi_output, $old_info) === false ? ' [' . $old_info . ']' : '');
+							}
+
+							if($info != null && stripos($info, $vend) === false)
+							{
+								$info = $vend . ' ' . $info;
+								break;
+							}
+						}
+					}
+				}
 			}
 
 			if(empty($info))
@@ -588,6 +612,29 @@ class phodevi_motherboard extends phodevi_device_interface
 			{
 				$info = phodevi_windows_parser::get_wmi_object('Win32_MotherboardDevice', 'Name');
 			}
+			foreach(array('Dell', 'Apple') as $vend)
+			{
+				if(stripos($info, $vend . ' ') !== false)
+				{
+					$wmi = shell_exec('wmic csproduct get name');
+					if(($x = strpos($wmi, 'Name')) !== false)
+					{
+						$wmi = trim(substr($wmi, $x + 4));
+
+						if(!empty($wmi) && stripos($wmi, 'System Product') === false)
+						{
+							if(stripos($wmi, $vend . ' ') === false)
+							{
+								$wmi = $vend . ' ' . $wmi;
+							}
+
+							$old_info = trim(str_ireplace(array($vend . ' ', 'Inc.'), '', $info));
+							$info = $wmi . (!empty($old_info) && strpos($wmi, $old_info) === false ? ' [' . $old_info . ']' : '');
+							break;
+						}
+					}
+				}
+			}
 		}
 
 		if((strpos($info, 'Mac ') !== false || strpos($info, 'MacBook') !== false) && strpos($info, 'Apple') === false)
