@@ -4,8 +4,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2008 - 2021, Phoronix Media
-	Copyright (C) 2008 - 2021, Michael Larabel
+	Copyright (C) 2008 - 2022, Phoronix Media
+	Copyright (C) 2008 - 2022, Michael Larabel
 	phodevi_system.php: The PTS Device Interface object for the system software
 
 	This program is free software; you can redistribute it and/or modify
@@ -59,6 +59,7 @@ class phodevi_system extends phodevi_device_interface
 			'security-features' => new phodevi_device_property('sw_security_features', phodevi::std_caching),
 			'kernel-extra-details' => new phodevi_device_property('sw_kernel_extra_details', phodevi::std_caching),
 			'battery' => new phodevi_device_property('battery', phodevi::smart_caching),
+			'platform-profile' => new phodevi_device_property('sw_platform_profile', phodevi::std_caching),
 			);
 	}
 	public static function sw_username()
@@ -76,6 +77,20 @@ class phodevi_system extends phodevi_device_interface
 
 		return $username;
 	}
+	public static function sw_platform_profile()
+	{
+		$platform_profile = '';
+
+		if(phodevi::is_linux())
+		{
+			if(is_file('/sys/firmware/acpi/platform_profile'))
+			{
+				$platform_profile = pts_file_io::file_get_contents('/sys/firmware/acpi/platform_profile');
+			}
+		}
+
+		return $platform_profile;
+	}
 	public static function sw_kernel_extra_details()
 	{
 		$extra = array();
@@ -382,10 +397,10 @@ class phodevi_system extends phodevi_device_interface
 		else if(phodevi::is_windows())
 		{
 			// TODO could use better detection to verify if C: or the desired disk under test... but most of the time will be NTFS anyways
-			$fs = filter_var(trim(shell_exec('powershell "(Get-WMIObject -Class Win32_Volume | Select DriveLetter,FreeSpace,Capacity,DeviceID,Label,@{Name=\"FileSystemType\";Expression={$_.\"FileSystem\"}})[1].FileSystemType"')),FILTER_SANITIZE_STRING, FILTER_FLAG_STRIP_HIGH);
+			$fs = filter_var(trim(shell_exec('powershell -NoProfile "(Get-WMIObject -Class Win32_Volume | Select DriveLetter,FreeSpace,Capacity,DeviceID,Label,@{Name=\"FileSystemType\";Expression={$_.\"FileSystem\"}})[1].FileSystemType"')), FILTER_UNSAFE_RAW, FILTER_FLAG_STRIP_HIGH);
 			if(empty($fs) || $fs == 'Unknown' || $fs == 'FAT32')
 			{
-				$fs = filter_var(trim(shell_exec('powershell "(Get-WMIObject -Class Win32_Volume | Select DriveLetter,FreeSpace,Capacity,DeviceID,Label,@{Name=\"FileSystemType\";Expression={$_.\"FileSystem\"}})[0].FileSystemType"')),FILTER_SANITIZE_STRING, FILTER_FLAG_STRIP_HIGH);
+				$fs = filter_var(trim(shell_exec('powershell -NoProfile "(Get-WMIObject -Class Win32_Volume | Select DriveLetter,FreeSpace,Capacity,DeviceID,Label,@{Name=\"FileSystemType\";Expression={$_.\"FileSystem\"}})[0].FileSystemType"')),FILTER_UNSAFE_RAW, FILTER_FLAG_STRIP_HIGH);
 			}
 
 			// Fallback for Windows 8
@@ -448,7 +463,7 @@ class phodevi_system extends phodevi_device_interface
 			{
 				$modinfo = trim(shell_exec('modinfo -F version vboxguest 2> /dev/null'));
 
-				if($modinfo != null && pts_strings::is_version(str_ireplace(array('_', 'RC', 'beta'), null, $modinfo)))
+				if($modinfo != null && pts_strings::is_version(str_ireplace(array('_', 'RC', 'beta'), '', $modinfo)))
 				{
 					$virtualized .= ' ' . $modinfo;
 				}
@@ -607,6 +622,7 @@ class phodevi_system extends phodevi_device_interface
 				$fc = file_get_contents($vuln);
 				$fc = str_replace('Mitigation: ', 'Mitigation of ', $fc);
 				$fc = str_replace('Speculative Store Bypass', 'SSB', $fc);
+				$fc = str_replace(' + ', ' plus ', $fc);
 				if(!empty($fc))
 				{
 					$security[] = basename($vuln) . ': ' . $fc;
@@ -637,7 +653,7 @@ class phodevi_system extends phodevi_device_interface
 		}
 		else if(phodevi::is_windows())
 		{
-			$mds_tool = getenv('USERPROFILE') . '\Downloads\mdstool-cli.exe';
+			$mds_tool = microsoft_dependency_handler::file_download_location() . 'mdstool-cli.exe';
 			if(is_file($mds_tool))
 			{
 				$mds_output = preg_replace('#\\x1b[[][^A-Za-z]*[A-Za-z]#', '', shell_exec($mds_tool));
@@ -650,6 +666,37 @@ class phodevi_system extends phodevi_device_interface
 					}
 				}
 			}
+
+			// Windows 10+ security features: VBS, HVCI
+			// https://docs.microsoft.com/en-us/windows/security/threat-protection/device-guard/enable-virtualization-based-protection-of-code-integrity#virtualizationbasedsecuritystatus
+			$vbs_status = trim(shell_exec('powershell -NoProfile "(Get-CimInstance -ErrorAction SilentlyContinue –ClassName Win32_DeviceGuard –Namespace root\Microsoft\Windows\DeviceGuard).VirtualizationBasedSecurityStatus"'));
+			switch($vbs_status) {
+				case '0':
+					$security[] = 'VBS: Disabled';
+					break;
+				case '1':
+					$security[] = 'VBS: Enabled but not running';
+					break;
+				case '2':
+					$security[] = 'VBS: Enabled and running';
+					break;
+			}
+
+			// https://docs.microsoft.com/en-us/windows/security/threat-protection/device-guard/enable-virtualization-based-protection-of-code-integrity#securityservicesconfigured
+			$security_services_running = preg_split('/\r\n|\n|\r/', trim(shell_exec('powershell -NoProfile "(Get-CimInstance -ErrorAction SilentlyContinue –ClassName Win32_DeviceGuard –Namespace root\Microsoft\Windows\DeviceGuard).SecurityServicesRunning"')));
+			if(in_array('2', $security_services_running)) {
+				$security[] = 'HVCI: Running';
+
+				// Mode Based Execution Control (MBEC) is relevant to HVCI performance and is available in Intel Kaby Lake and newer and AMD Zen 2 and newer
+				// https://docs.microsoft.com/en-us/windows/security/threat-protection/device-guard/enable-virtualization-based-protection-of-code-integrity#hvci-features
+				// https://docs.microsoft.com/en-us/windows/security/threat-protection/device-guard/enable-virtualization-based-protection-of-code-integrity#availablesecurityproperties
+				$available_security_properties = preg_split('/\r\n|\n|\r/', trim(shell_exec('powershell -NoProfile "(Get-CimInstance -ErrorAction SilentlyContinue –ClassName Win32_DeviceGuard –Namespace root\Microsoft\Windows\DeviceGuard).AvailableSecurityProperties"')));
+				if(in_array('7', $available_security_properties)) {
+					$security[] = 'MBEC: Available';
+				} else {
+					$security[] = 'MBEC: Unavailable';
+				}
+			}
 		}
 
 		return !empty($security) ? implode(' + ',  $security) : null;
@@ -727,33 +774,11 @@ class phodevi_system extends phodevi_device_interface
 			$v = trim(shell_exec('pgcc --version 2>&1'));
 			$v = substr($v, strpos($v, 'pgcc ') + 5);
 			$v = substr($v, 0, strpos($v, ' '));
-			if(pts_strings::is_version(str_replace('-', null, $v)))
+			if(pts_strings::is_version(str_replace('-', '', $v)))
 			{
 				$compilers['pgcc'] .= ' ' . $v;
 			}
 		}
-		if(pts_client::executable_in_path('opencc'))
-		{
-			// Open64
-			$compilers['opencc'] = 'Open64 ' . trim(shell_exec('opencc -dumpversion 2>&1'));
-		}
-
-		if(pts_client::executable_in_path('pathcc'))
-		{
-			// PathCC / EKOPath / PathScale Compiler Suite
-			$compilers['pathcc'] = 'PathScale ' . trim(shell_exec('pathcc -dumpversion 2>&1'));
-		}
-
-		if(pts_client::executable_in_path('tcc'))
-		{
-			// TCC - Tiny C Compiler
-			$tcc = explode(' ', trim(shell_exec('tcc -v 2>&1')));
-
-			if($tcc[1] == 'version')
-			{
-				$compilers['opencc'] = 'TCC ' . $tcc[2];
-			}
-		}
 
 		if(pts_client::executable_in_path('pcc'))
 		{
@@ -825,6 +850,24 @@ class phodevi_system extends phodevi_device_interface
 			$compilers['clang'] = $compiler_info;
 		}
 
+		// For now at least Intel oneAPI has symlink from clang to icpx so this below code isn't needed in such case
+		if(!pts_client::executable_in_path('clang') && ($icpx = pts_client::executable_in_path('icpx')))
+		{
+			// Intel oneAPI DPC++/C++ Compiler
+			$icpx = shell_exec(escapeshellarg($icpx) . ' --version');
+			$icpx = substr($icpx, 0, strpos($icpx, PHP_EOL));
+			if(stripos($icpx, 'oneAPI') !== false)
+			{
+				$icpx = str_ireplace(array('(R)'), '', $icpx);
+				if(($x = strpos($icpx, ' (')) !== false)
+				{
+					$icpx = substr($icpx, 0, $x);
+				}
+			}
+
+			$compilers['icpx'] = $icpx;
+		}
+
 		if(($llvm_ld = pts_client::executable_in_path('llvm-link')) || ($llvm_ld = pts_client::executable_in_path('llvm-ld')))
 		{
 			// LLVM - Low Level Virtual Machine
@@ -838,7 +881,7 @@ class phodevi_system extends phodevi_device_interface
 				$info = substr($info, 0, strpos($info, PHP_EOL, $s));
 				$info = substr($info, (strrpos($info, ' ') + 1));
 
-				if(pts_strings::is_version(str_replace('svn', null, $info)))
+				if(pts_strings::is_version(str_replace('svn', '', $info)))
 				{
 					$compilers['llvmc'] = 'LLVM ' . $info;
 				}
@@ -848,7 +891,7 @@ class phodevi_system extends phodevi_device_interface
 		{
 			// LLVM - Low Level Virtual Machine config
 			$info = trim(shell_exec('llvm-config --version 2> /dev/null'));
-			if(pts_strings::is_version(str_replace('svn', null, $info)))
+			if(pts_strings::is_version(str_replace('svn', '', $info)))
 			{
 				$compilers['llvmc'] = 'LLVM ' . $info;
 			}
@@ -911,10 +954,22 @@ class phodevi_system extends phodevi_device_interface
 			}
 		}
 
-		if(pts_client::executable_in_path('icc'))
+		if(($icc = pts_client::executable_in_path('icc')) || ($icc = pts_client::executable_in_path('icpc')))
 		{
-			// Intel C++ Compiler
-			$compilers['icc'] = 'ICC';
+			// Intel oneAPI DPC++/C++ Compiler
+			$icc = shell_exec(escapeshellarg($icc) . ' --version');
+			$icc = substr($icc, 0, strpos($icc, PHP_EOL));
+			if(stripos($icc, 'icc') !== false)
+			{
+				$icc = str_ireplace(array('(R)', '(ICC)', '(C)'), '', $icc);
+				if(($x = strpos($icpx, ' (')) !== false)
+				{
+					$icpx = substr($icc, 0, $x);
+				}
+			}
+
+			$icc = str_replace('icc', 'ICC', $icc);
+			$compilers['icc'] = $icc;
 		}
 
 		if(phodevi::is_macos() && pts_client::executable_in_path('xcodebuild'))
@@ -935,7 +990,7 @@ class phodevi_system extends phodevi_device_interface
 			$nvcc = shell_exec($nvcc . ' --version 2>&1');
 			if(($s = strpos($nvcc, 'release ')) !== false)
 			{
-				$nvcc = str_replace(array(','), null, substr($nvcc, ($s + 8)));
+				$nvcc = str_replace(array(','), '', substr($nvcc, ($s + 8)));
 				$nvcc = substr($nvcc, 0, strpos($nvcc, ' '));
 
 				if(pts_strings::is_version($nvcc))
@@ -946,9 +1001,9 @@ class phodevi_system extends phodevi_device_interface
 		}
 
 		// Try to make the compiler that's used by default to appear first
-		if(pts_client::read_env('CC') && isset($compilers[basename(pts_strings::first_in_string(pts_client::read_env('CC'), ' '))]))
+		if(getenv('CC') && isset($compilers[basename(pts_strings::first_in_string(getenv('CC'), ' '))]))
 		{
-			$cc_env = basename(pts_strings::first_in_string(pts_client::read_env('CC'), ' '));
+			$cc_env = basename(pts_strings::first_in_string(getenv('CC'), ' '));
 			$default_compiler = $compilers[$cc_env];
 			unset($compilers[$cc_env]);
 			array_unshift($compilers, $default_compiler);
@@ -960,7 +1015,7 @@ class phodevi_system extends phodevi_device_interface
 			if(isset($compilers[$cc_link]))
 			{
 				$default_compiler = $compilers[$cc_link];
-				unset($compilers[pts_client::read_env('CC')]);
+				unset($compilers[getenv('CC')]);
 				array_unshift($compilers, $default_compiler);
 			}
 		}
@@ -995,7 +1050,37 @@ class phodevi_system extends phodevi_device_interface
 	}
 	public static function sw_kernel()
 	{
-		return php_uname('r');
+		if(phodevi::is_windows())
+		{
+			// CurrentBuild and CurrentVersion are available since at least NT 4.0
+			// CurrentVersion is frozen at 6.3 (same as Windows 8.1) in Windows 10 & 11
+			$current_build = trim(shell_exec('powershell -NoProfile "If (Get-ItemProperty -ErrorAction SilentlyContinue -Path \'Registry::HKEY_LOCAL_MACHINE\Software\Microsoft\Windows NT\CurrentVersion\' CurrentBuild) { (Get-ItemProperty -Path \'Registry::HKEY_LOCAL_MACHINE\Software\Microsoft\Windows NT\CurrentVersion\' CurrentBuild).CurrentBuild } Else { $null }"'));
+
+			// Windows 10 and later add CurrentMajorVersionNumber, CurrentMinorVersionNumber and UBR
+			$current_major_version_number = trim(shell_exec('powershell -NoProfile "If (Get-ItemProperty -ErrorAction SilentlyContinue -Path \'Registry::HKEY_LOCAL_MACHINE\Software\Microsoft\Windows NT\CurrentVersion\' CurrentMajorVersionNumber) { (Get-ItemProperty -Path \'Registry::HKEY_LOCAL_MACHINE\Software\Microsoft\Windows NT\CurrentVersion\' CurrentMajorVersionNumber).CurrentMajorVersionNumber } Else { $null }"'));
+
+			// Try using Windows 10+ values
+			if (is_numeric($current_major_version_number)) {
+				$current_minor_version_number = trim(shell_exec('powershell -NoProfile "If (Get-ItemProperty -ErrorAction SilentlyContinue -Path \'Registry::HKEY_LOCAL_MACHINE\Software\Microsoft\Windows NT\CurrentVersion\' CurrentMinorVersionNumber) { (Get-ItemProperty -Path \'Registry::HKEY_LOCAL_MACHINE\Software\Microsoft\Windows NT\CurrentVersion\' CurrentMinorVersionNumber).CurrentMinorVersionNumber } Else { $null }"'));
+				$update_build_revision = trim(shell_exec('powershell -NoProfile "If (Get-ItemProperty -ErrorAction SilentlyContinue -Path \'Registry::HKEY_LOCAL_MACHINE\Software\Microsoft\Windows NT\CurrentVersion\' UBR) { (Get-ItemProperty -Path \'Registry::HKEY_LOCAL_MACHINE\Software\Microsoft\Windows NT\CurrentVersion\' UBR).UBR } Else { $null }"'));
+
+				if(is_numeric($current_minor_version_number) && is_numeric($current_build) && is_numeric($update_build_revision)) {
+					return $current_major_version_number . '.' . $current_minor_version_number . '.' . $current_build . '.' . $update_build_revision;
+				}
+			}
+
+			// Fall back to Windows 8.1 and earlier values
+			$current_version = trim(shell_exec('powershell -NoProfile "If (Get-ItemProperty -ErrorAction SilentlyContinue -Path \'Registry::HKEY_LOCAL_MACHINE\Software\Microsoft\Windows NT\CurrentVersion\' CurrentVersion) { (Get-ItemProperty -Path \'Registry::HKEY_LOCAL_MACHINE\Software\Microsoft\Windows NT\CurrentVersion\' CurrentVersion).CurrentVersion } Else { $null }"'));
+
+			if(is_numeric($current_version) && is_numeric($current_build)) {
+				return $current_version . '.' . $current_build;
+			}
+
+			// Fall back to PHP implementation
+			return php_uname('r');
+		} else {
+			return php_uname('r');
+		}
 	}
 	public static function sw_kernel_parameters()
 	{
@@ -1162,7 +1247,7 @@ class phodevi_system extends phodevi_device_interface
 				$vendor = substr($vendor, 0, $x);
 			}
 
-			$vendor = str_replace(array(' Software'), null, $vendor);
+			$vendor = str_replace(array(' Software'), '', $vendor);
 		}
 		else if(phodevi::is_hurd())
 		{
@@ -1287,7 +1372,7 @@ class phodevi_system extends phodevi_device_interface
 		$desktop = null;
 		$desktop_environment = null;
 		$desktop_version = null;
-		$desktop_session = pts_client::read_env('DESKTOP_SESSION');
+		$desktop_session = getenv('DESKTOP_SESSION');
 
 		if(pts_client::is_process_running('gnome-shell'))
 		{
@@ -1523,7 +1608,7 @@ class phodevi_system extends phodevi_device_interface
 			}
 			if(pts_client::is_process_running('unity-system-compositor'))
 			{
-				$unity_system_comp = trim(str_replace('unity-system-compositor', null, shell_exec('unity-system-compositor --version 2>&1')));
+				$unity_system_comp = trim(str_replace('unity-system-compositor', '', shell_exec('unity-system-compositor --version 2>&1')));
 
 				if(pts_strings::is_version($unity_system_comp))
 				{
@@ -1716,7 +1801,7 @@ class phodevi_system extends phodevi_device_interface
 						{
 							// This path works for at least finding NVIDIA Tegra 2 DDX (via tegra_fb)
 							$display_driver = file_get_contents('/sys/class/graphics/fb0/name');
-							$display_driver = str_replace(array('drm', '_fb'), null, $display_driver);
+							$display_driver = str_replace(array('drm', '_fb'), '', $display_driver);
 							$driver_version = phodevi_parser::read_xorg_module_version($display_driver . '_drv');
 						}
 						break;
@@ -1785,7 +1870,12 @@ class phodevi_system extends phodevi_device_interface
 			{
 				$llvm = substr($renderer, $s);
 				$llvm = substr($llvm, 0, strpos($llvm, ')'));
-				$info .= ' (' . $llvm . ')';
+				if(!empty($llvm) && phodevi::read_property('system', 'kernel'))
+				{
+					// Mesa Oibaf PPA for example includes kernel version but not important since PTS already reports kernel
+					$llvm = str_replace(array(',', phodevi::read_property('system', 'kernel')), '', $llvm);
+				}
+				$info .= ' (' . trim($llvm) . ')';
 			}
 		}
 
@@ -1859,7 +1949,7 @@ class phodevi_system extends phodevi_device_interface
 		{
 			$info = shell_exec('glxinfo 2>&1 | grep vendor');
 
-			if(($pos = strpos($info, 'OpenGL vendor string:')) !== false)
+			if($info && ($pos = strpos($info, 'OpenGL vendor string:')) !== false)
 			{
 				$info = substr($info, $pos + 22);
 				$info = trim(substr($info, 0, strpos($info, "\n")));
@@ -2084,9 +2174,9 @@ class phodevi_system extends phodevi_device_interface
 		{
 			$wine_version = trim(shell_exec('wine --version 2>&1'));
 		}
-		else if(pts_client::executable_in_path('winecfg.exe') != false && pts_client::read_env('WINE_VERSION'))
+		else if(pts_client::executable_in_path('winecfg.exe') != false && getenv('WINE_VERSION'))
 		{
-			$wine_version = trim(pts_client::read_env('WINE_VERSION'));
+			$wine_version = trim(getenv('WINE_VERSION'));
 
 			if(stripos($wine_version, 'wine') === false)
 			{
