@@ -156,7 +156,11 @@ class pts_math
 	public static function get_precision($number)
 	{
 		// number of decimal digits
-		if(is_array($number))
+		if($number === null)
+		{
+			return 0;
+		}
+		else if(is_array($number))
 		{
 			$max_precision = 0;
 			foreach($number as $n)
@@ -223,7 +227,7 @@ class pts_math
 		{
 			return $values[$index];
 		}
-		else
+		else if(is_numeric($values[($index - 1)]) && is_numeric($values[($index + 1)]))
 		{
 			return ($values[($index - 1)] + $values[($index + 1)]) / 2;
 		}
