@@ -127,7 +127,18 @@ class pts_render
 		// NOTICE: $save_as doesn't appear used anymore
 		self::attribute_processing_on_result_object($result_object, $result_file, $extra_attributes);
 
-		if($result_object->test_result_buffer->get_count() == 0)
+		$has_a_result = false;
+		foreach($result_object->test_result_buffer->buffer_items as &$buffer_item)
+		{
+			// Check to make sure at least one result is not null/empty so there is something to render for the graph
+			// i.e. make sure not all of the results failed to run for this result object
+			if($buffer_item->get_result_value() != null)
+			{
+				$has_a_result = true;
+				break;
+			}
+		}
+		if(!$has_a_result)
 		{
 			return false;
 		}
@@ -583,7 +594,8 @@ class pts_render
 		$selected_renderer = 'SVG';
 
 		// Yahoo Slurp, msnbot, and googlebot should always be served SVG so no problems there
-
+		// 2021: all relatively recent browsers of thep ast decade should be fine...
+		/*
 		if(($p = strpos($user_agent, 'Gecko/')) !== false)
 		{
 			// Mozilla Gecko-based browser (Firefox, etc)
@@ -646,6 +658,7 @@ class pts_render
 			// Facebook uses this string for its Like/Share crawler, so serve it a PNG so it can use it as an image
 			$selected_renderer = 'PNG';
 		}
+		*/
 
 		return $selected_renderer;
 	}
