@@ -217,6 +217,10 @@ class pts_result_file_analyzer
 					continue;
 				}
 				$result = $iv_map[$identifier];
+				if(!is_numeric($result) || !is_numeric($value))
+				{
+					continue;
+				}
 
 				if($result_object->test_profile->get_result_proportion() == 'HIB')
 				{
@@ -293,10 +297,9 @@ class pts_result_file_analyzer
 						{
 							$avg_time = round(array_sum($test_run_times) / count($test_run_times));
 							// Cost-perf-per-hour calculation, e.g. cloud costs...
-							// self::$TEST_RUN_TIME_ELAPSED is seconds run.....
 							$cost_to_run_test = round(($value / 60 / 60) * $avg_time, 3);
 
-							if($cost_to_run_test < 0.01)
+							if($cost_to_run_test < 0.001)
 							{
 								continue;
 							}
@@ -421,6 +424,10 @@ class pts_result_file_analyzer
 				$prev_buffer = null;
 				foreach($geo_mean_result->test_result_buffer->get_buffer_items() as $bi)
 				{
+					if(!is_numeric($bi->get_result_value()))
+					{
+						continue;
+					}
 					if($prev_buffer == null)
 					{
 						$prev_buffer = $bi;
@@ -500,7 +507,7 @@ class pts_result_file_analyzer
 	}
 	public static function generate_wins_losses_results($result_file, $only_wins = false)
 	{
-		$results = null;
+		$results = array();
 		$result_file_identifiers_count = $result_file->get_system_count();
 		$wins = array();
 		$losses = array();
@@ -519,9 +526,13 @@ class pts_result_file_analyzer
 				continue;
 			}
 
-			$tests_counted++;
 			$winner = $result->get_result_first();
 			$loser = $result->get_result_last();
+			if($winner == null || $loser == null)
+			{
+				continue;
+			}
+			$tests_counted++;
 
 			if(!isset($wins[$winner]))
 			{
@@ -544,7 +555,7 @@ class pts_result_file_analyzer
 
 		if(empty($wins) || empty($losses))
 		{
-			return;
+			return array();
 		}
 
 		arsort($wins);
@@ -662,14 +673,23 @@ class pts_result_file_analyzer
 					}
 				}
 			}
-
+			$skip_this = false;
 			foreach($result->test_result_buffer->get_buffer_items() as $buffer_item)
 			{
 				$r = $buffer_item->get_result_value();
 				if(!is_numeric($r) || $r == 0)
 				{
-					continue;
+					$skip_this = true;
+					break;
 				}
+			}
+			if($skip_this)
+			{
+				continue;
+			}
+			foreach($result->test_result_buffer->get_buffer_items() as $buffer_item)
+			{
+				$r = $buffer_item->get_result_value();
 				if($result->test_profile->get_result_proportion() == 'LIB')
 				{
 					// convert to HIB
@@ -709,6 +729,7 @@ class pts_result_file_analyzer
 			$test_result->set_used_arguments_description('Result Composite' . ($result_file->get_title() != null ? ' - ' . $result_file->get_title() : null));
 			$test_result->set_used_arguments('Geometric-Mean');
 			$test_result->test_result_buffer = new pts_test_result_buffer();
+
 			foreach($results as $identifier => $values)
 			{
 				if(count($values) < 2)
@@ -801,6 +822,10 @@ class pts_result_file_analyzer
 
 		foreach($results as $test_title => $test_results)
 		{
+			if(count($test_results) != $system_count)
+			{
+				continue;
+			}
 			$test_profile = new pts_test_profile();
 			$test_result = new pts_test_result($test_profile);
 			$test_result->test_profile->set_test_title($test_title);
@@ -856,6 +881,22 @@ class pts_result_file_analyzer
 				// Harmonic mean is relevant for tests of rates, MB/s, FPS, ns/day, etc.
 				continue;
 			}
+			$skip_this_result = false;
+			foreach($result->test_result_buffer->get_buffer_items() as $buffer_item)
+			{
+				$rv = $buffer_item->get_result_value();
+				if(!is_numeric($rv))
+				{
+					$skip_this_result = true;
+					break;
+				}
+			}
+
+			if($skip_this_result)
+			{
+				continue;
+			}
+
 			foreach($result->test_result_buffer->get_buffer_items() as $buffer_item)
 			{
 				$ri = $buffer_item->get_result_identifier();
@@ -969,10 +1010,14 @@ class pts_result_file_analyzer
 				continue;
 			}
 
-			$tests_counted++;
 			$winner = $result->get_result_first();
 			$loser = $result->get_result_last();
 
+			if($winner == null || $loser == null)
+			{
+				continue;
+			}
+
 			if(!isset($wins[$winner]))
 			{
 				$wins[$winner] = 1;
@@ -990,6 +1035,7 @@ class pts_result_file_analyzer
 			{
 				$losses[$loser]++;
 			}
+			$tests_counted++;
 		}
 
 		if(empty($wins) || empty($losses))
@@ -1479,6 +1525,12 @@ class pts_result_file_analyzer
 				unset($json['cpu-scaling-governor']);
 			}
 
+			if(!empty($json['platform-profile']))
+			{
+				$cpu_data[] = 'Platform Profile: ' . $json['platform-profile'];
+				unset($json['platform-profile']);
+			}
+
 			if(!empty($json['cpu-microcode']))
 			{
 				$cpu_data[] = 'CPU Microcode: ' . $json['cpu-microcode'];
@@ -1504,7 +1556,7 @@ class pts_result_file_analyzer
 			$system_attributes['Processor'][$identifier] = 'SMT (threads per core): ' . $json['cpu-smt'];
 			unset($json['cpu-smt']);
 		}
-		if(isset($json['graphics-2d-acceleration']) || isset($json['graphics-aa']) || isset($json['graphics-af']))
+		if(isset($json['graphics-2d-acceleration']) || isset($json['graphics-aa']) || isset($json['graphics-af']) || isset($json['bar1-visible-vram']))
 		{
 			$report = array();
 			foreach(array('graphics-2d-acceleration', 'graphics-aa', 'graphics-af') as $check)
@@ -1515,6 +1567,11 @@ class pts_result_file_analyzer
 					unset($json[$check]);
 				}
 			}
+			if(isset($json['bar1-visible-vram']))
+			{
+				$report[] = 'BAR1 / Visible vRAM Size: ' . $json['bar1-visible-vram'];
+				unset($json['bar1-visible-vram']);
+			}
 			$system_attributes['Graphics'][$identifier] = implode(' - ' , $report);
 		}
 		if(isset($json['graphics-compute-cores']))
