@@ -40,7 +40,6 @@ class pts_test_result
 
 	// Added to make it easy to have PTS modules run a custom binary prior to running a program for the test
 	public $exec_binary_prepend = null;
-	public $test_result_standard_output = null;
 
 	protected $already_normalized = false;
 	public $dynamically_generated = false;
@@ -247,7 +246,7 @@ class pts_test_result
 				$tp = $this->test_profile->get_title();
 			}
 
-			return pts_test_profile::generate_comparison_hash($tp, $this->get_arguments(), null, null, $raw_output);
+			return pts_test_profile::generate_comparison_hash($tp, $this->get_arguments(), '', '', $raw_output);
 		}
 	}
 	public function __toString()
@@ -290,6 +289,10 @@ class pts_test_result
 			$divide_value = 0;
 			foreach($keys as $k)
 			{
+				if(!is_numeric($this->test_result_buffer->buffer_items[$k]->get_result_value()))
+				{
+					continue;
+				}
 				if($divide_value == 0)
 				{
 					$divide_value = $this->test_result_buffer->buffer_items[$k]->get_result_value();
@@ -398,7 +401,7 @@ class pts_test_result
 		}
 
 		$spread = $best / $worst;
-		if($this->test_profile->get_result_proportion() == 'LIB')
+		if($this->test_profile->get_result_proportion() == 'LIB' && $spread != 0)
 		{
 			$spread = 1 / $spread;
 		}
@@ -443,6 +446,13 @@ class pts_test_result
 
 		foreach($key_sets as $keys)
 		{
+			foreach($keys as $i => $k)
+			{
+				if(!is_numeric($this->test_result_buffer->buffer_items[$k]->get_result_value()))
+				{
+					unset($keys[$i]);
+				}
+			}
 			if($this->test_profile->get_result_proportion() == 'LIB')
 			{
 				// Invert values for LIB
@@ -489,7 +499,7 @@ class pts_test_result
 				}
 			}
 
-			if($divide_value != 0)
+			if($divide_value != 0 && is_numeric($divide_value))
 			{
 				foreach($keys as $k)
 				{
@@ -551,6 +561,11 @@ class pts_test_result
 			$remove_set = true;
 			foreach($keys as $k)
 			{
+				if(!is_numeric($this->test_result_buffer->buffer_items[$k]->get_result_value()))
+				{
+					continue;
+				}
+
 				if($base_value == -1)
 				{
 					$base_value = $this->test_result_buffer->buffer_items[$k]->get_result_value();
@@ -767,7 +782,7 @@ class pts_test_result
 			{
 				$this_value = $this->test_result_buffer->buffer_items[$k]->get_result_value();
 				$this_id = $this->test_result_buffer->buffer_items[$k]->get_result_identifier();
-				if($prev_value != -1 && $prev_id != -1)
+				if($prev_value != -1 && $prev_id != -1 && is_numeric($this_value))
 				{
 					$d = abs(($prev_value / $this_value) - 1);
 					if($d > $threshold_level)
