@@ -199,7 +199,7 @@ class pts_test_result_parser
 		}
 
 		$extra_arguments = $test_run_request->get_arguments();
-		$pts_test_arguments = trim($test_run_request->test_profile->get_default_arguments() . ' ' . str_replace($test_run_request->test_profile->get_default_arguments(), '', $extra_arguments) . ' ' . $test_run_request->test_profile->get_default_post_arguments());
+		$pts_test_arguments = trim($test_run_request->test_profile->get_default_arguments() . ' ' . ($test_run_request->test_profile->get_default_arguments() != null && !empty($extra_arguments) ? str_replace($test_run_request->test_profile->get_default_arguments(), '', $extra_arguments) : $extra_arguments) . ' ' . $test_run_request->test_profile->get_default_post_arguments());
 
 		switch($test_run_request->test_profile->get_display_format())
 		{
@@ -635,14 +635,14 @@ class pts_test_result_parser
 					{
 						pts_test_result_parser::debug_message('Result Parsing Line Before Hint: ' . $line_before_hint);
 						$line = substr($output, strpos($output, "\n", strrpos($output, $line_before_hint)));
-						$line = substr($line, 0, strpos($line, "\n", 1));
+						$line = substr($line, 0, strpos($line, "\n", (isset($line[2]) ? 1 : 0)));
 						$output = substr($output, 0, strrpos($output, "\n", strrpos($output, $line_before_hint))) . "\n";
 					}
 					else if($line_after_hint != null)
 					{
 						pts_test_result_parser::debug_message('Result Parsing Line After Hint: ' . $line_after_hint);
 						$line = substr($output, 0, strrpos($output, "\n", strrpos($output, $line_after_hint)));
-						$line = substr($line, strrpos($line, "\n", 1) + 1);
+						$line = substr($line, strrpos($line, "\n", (isset($line[2]) ? 1 : 0)) + 1);
 						$output = null;
 					}
 					else if($search_key != null)
@@ -675,7 +675,7 @@ class pts_test_result_parser
 						pts_test_result_parser::debug_message('No Result Parsing Hint, Including Entire Result Output');
 						$line = trim($output);
 					}
-					if($e->get_turn_chars_to_space() != null)
+					if($e->get_turn_chars_to_space() != null && $line != null)
 					{
 						$line = str_replace($e->get_turn_chars_to_space(), ' ', $line);
 					}
@@ -687,7 +687,7 @@ class pts_test_result_parser
 					do
 					{
 						$try_again = false;
-						$r = explode(' ', pts_strings::trim_spaces(str_replace($space_out_chars, ' ', str_replace('=', ' = ', $line))));
+						$r = $line != null ? explode(' ', pts_strings::trim_spaces(str_replace($space_out_chars, ' ', str_replace('=', ' = ', $line)))) : array();
 						$r_pos = array_search($key_for_result, $r);
 
 						if($e->get_result_before_string() != null)
@@ -738,7 +738,7 @@ class pts_test_result_parser
 						else
 						{
 							// POSSIBLE FALLBACKS TO TRY AGAIN
-							if(!$did_try_colon_fallback && strpos($line, ':') !== false)
+							if(!$did_try_colon_fallback && $line != null && strpos($line, ':') !== false)
 							{
 								$line = str_replace(':', ': ', $line);
 								$did_try_colon_fallback = true;
@@ -861,7 +861,7 @@ class pts_test_result_parser
 
 		if($is_pass_fail_test)
 		{
-			if(str_replace(array('PASS', 'FAIL', ','), null, $test_result) == null)
+			if(str_replace(array('PASS', 'FAIL', ','), '', $test_result) == null)
 			{
 				// already a properly formatted multi-pass fail
 			}
@@ -954,7 +954,7 @@ class pts_test_result_parser
 				return true;
 			}
 		}
-		else if(strpos($numeric_input, ':') !== false && strpos($numeric_input, '.') !== false && is_numeric(str_replace(array(':', '.'), null, $numeric_input)) && stripos($line, 'time') !== false)
+		else if(strpos($numeric_input, ':') !== false && strpos($numeric_input, '.') !== false && is_numeric(str_replace(array(':', '.'), '', $numeric_input)) && stripos($line, 'time') !== false)
 		{
 			// Convert e.g. 03:03.17 to seconds, relevant for at least pts/blender
 			$seconds = 0;
@@ -1011,7 +1011,7 @@ class pts_test_result_parser
 	{
 		if($e->get_strip_from_result() != null)
 		{
-			$test_result = str_replace($e->get_strip_from_result(), null, $test_result);
+			$test_result = str_replace($e->get_strip_from_result(), '', $test_result);
 		}
 		if($e->get_strip_result_postfix() != null && substr($test_result, 0 - strlen($e->get_strip_result_postfix())) == $e->get_strip_result_postfix())
 		{
