@@ -3,8 +3,8 @@
 /*
 	Phoronix Test Suite
 	URLs: http://www.phoronix.com, http://www.phoronix-test-suite.com/
-	Copyright (C) 2008 - 2021, Phoronix Media
-	Copyright (C) 2008 - 2021, Michael Larabel
+	Copyright (C) 2008 - 2022, Phoronix Media
+	Copyright (C) 2008 - 2022, Michael Larabel
 	pts-core.php: To boot-strap the Phoronix Test Suite start-up
 
 	This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,7 @@ class pts_core
 {
 	public static function init()
 	{
-		set_time_limit(0);
+		//set_time_limit(0);
 		pts_define_directories(); // Define directories
 
 		pts_define('PTS_INIT_TIME', time());
@@ -48,13 +48,13 @@ class pts_core
 				$userinfo = posix_getpwuid(posix_getuid());
 				$userhome = $userinfo['dir'];
 			}
-			else if(($home = pts_client::read_env('HOME')))
+			else if(($home = getenv('HOME')))
 			{
 				$userhome = $home;
 			}
-			else if(($home = pts_client::read_env('HOMEPATH')))
+			else if(($home = getenv('HOMEPATH')))
 			{
-				$userhome = pts_client::read_env('HOMEDRIVE') . $home;
+				$userhome = getenv('HOMEDRIVE') . $home;
 			}
 			else if(PTS_IS_DAEMONIZED_SERVER_PROCESS)
 			{
@@ -82,15 +82,10 @@ class pts_core
 
 		return $userhome;
 	}
-	public static function codename($full_string = false)
+	public static function program_title()
 	{
-		$codename = ucwords(strtolower(PTS_CODENAME));
-
-		return ($full_string ? 'PhoronixTestSuite/' : null) . $codename;
-	}
-	public static function program_title($show_codename = false)
-	{
-		return 'Phoronix Test Suite v' . PTS_VERSION . ($show_codename ? ' (' . pts_core::codename() . ')' : null);
+		// First argument was originally $show_codename, but no longer used/honored
+		return 'Phoronix Test Suite v' . PTS_VERSION;
 	}
 }
 function pts_define($name, $value = null)
@@ -232,79 +227,11 @@ function pts_needed_extensions()
 		array(0, function_exists('readline'), 'Readline', 'Readline support is useful for tab-based auto-completion support.'),
 		);
 }
-function pts_version_codenames()
-{
-	// Lots of inspiration for codenames often from Norwegian municipalities
-	// https://en.wikipedia.org/wiki/List_of_municipalities_of_Norway
-	return array(
-		// Sør-Trøndelag - Norway
-		'1.0' => 'Trondheim',
-		'1.2' => 'Malvik',
-		'1.4' => 'Orkdal',
-		'1.6' => 'Tydal',
-		'1.8' => 'Selbu',
-		// Troms - Norway
-		'2.0' => 'Sandtorg',
-		'2.2' => 'Bardu',
-		'2.4' => 'Lenvik',
-		'2.6' => 'Lyngen',
-		'2.8' => 'Torsken',
-		// Aust-Agder - Norway
-		'2.9' => 'Iveland', // early PTS3 development work
-		'3.0' => 'Iveland',
-		'3.2' => 'Grimstad',
-		'3.4' => 'Lillesand',
-		'3.6' => 'Arendal',
-		'3.8' => 'Bygland',
-		// Rogaland - Norway
-		'4.0' => 'Suldal',
-		'4.2' => 'Randaberg',
-		'4.4' => 'Forsand',
-		'4.6' => 'Utsira',
-		'4.8' => 'Sokndal',
-		// Tulskaya oblast / Tula Oblast region - Russia
-		'5.0' => 'Plavsk',
-		'5.2' => 'Khanino',
-		'5.4' => 'Lipki',
-		'5.6' => 'Dedilovo',
-		'5.8' => 'Belev',
-		// Finnmark - Norway
-		'6.0' => 'Hammerfest',
-		'6.2' => 'Gamvik',
-		'6.4' => 'Hasvik',
-		'6.6' => 'Loppa',
-		'6.8' => 'Tana',
-		// Hedmark - Norway
-		'7.0' => 'Ringsaker',
-		'7.2' => 'Trysil',
-		'7.4' => 'Tynset',
-		'7.6' => 'Alvdal',
-		'7.8' => 'Folldal',
-		// Østfold - Norway
-		'8.0' => 'Aremark',
-		'8.2' => 'Rakkestad',
-		'8.4' => 'Skiptvet',
-		'8.6' => 'Spydeberg',
-		'8.8' => 'Hvaler',
-		// Akershus - Norway
-		'9.0' => 'Asker',
-		'9.2' => 'Hurdal',
-		'9.4' => 'Vestby',
-		'9.6' => 'Nittedal',
-		'9.8' => 'Nesodden',
-		//  Senja Municipality in Troms og Finnmark - Norway
-		'10.0' => 'Finnsnes',
-		'10.2' => 'Harstad',
-		'10.4' => 'Ibestad',
-		'10.6' => 'Tjeldsund',
-		'10.8' => 'Nesseby',
-		);
-}
 
-pts_define('PTS_VERSION', '10.4.0');
-pts_define('PTS_CORE_VERSION', 10400);
-pts_define('PTS_RELEASE_DATE', '20210516');
-pts_define('PTS_CODENAME', 'Ibestad');
+pts_define('PTS_VERSION', '10.8.3');
+pts_define('PTS_CORE_VERSION', 10830);
+pts_define('PTS_RELEASE_DATE', '20220423');
+pts_define('PTS_CODENAME', 'Nesseby');
 
 pts_define('PTS_IS_CLIENT', (defined('PTS_MODE') && strstr(PTS_MODE, 'CLIENT') !== false));
 pts_define('PTS_IS_WEB_CLIENT', (defined('PTS_MODE') && PTS_MODE == 'WEB_CLIENT'));
@@ -371,7 +298,7 @@ if(PTS_IS_CLIENT && ini_get('date.timezone') == null)
 	}
 	else if(strtoupper(substr(PHP_OS, 0, 3)) == 'WIN')
 	{
-		$tz = trim(shell_exec('powershell "(Get-TimeZone).BaseUtcOffset.Hours"'));
+		$tz = trim(shell_exec('powershell -NoProfile "(Get-TimeZone).BaseUtcOffset.Hours"'));
 		$tz = is_numeric($tz) ? timezone_name_from_abbr('', ($tz * 60 * 60), 0) : null;
 	}
 
